﻿Imports System.IO.Ports

Public Class Form1

    Private WithEvents sockServer As TcpServer
    Private WithEvents sockClient As TcpClient
    Private WithEvents ComPort As SerialPort = New SerialPort

    Private Sub Form_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Text = AppTitleAndVersion()
        InitAllCombo()
        Load_INI()
        ComPort_Init()
        EventsAreEnabled = True
    End Sub

    Private Sub Form_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        Save_INI()
        If ComPort.IsOpen() Then
            ComPort.DiscardInBuffer()
        End If
        If sockClient IsNot Nothing Then
            sockClient.StopClient()
            sockClient = Nothing
        End If
        If sockServer IsNot Nothing Then
            sockServer.StopListener()
            sockServer = Nothing
        End If
    End Sub

    ' LIVIO
    ' ===========================================================================================
    '  INIT ALL COMBO
    ' ===========================================================================================
    Private Sub InitAllCombo()
        InitComboPortNames()
        InitComboDatabits()
        InitComboParity()
        InitComboStopBits()
        InitComboHandshake()
    End Sub
    Private Sub InitComboPortNames()
        cmb_PortName.Items.Clear()
        For Each s As String In SerialPort.GetPortNames()
            cmb_PortName.Items.Add(s)
        Next
        If cmb_PortName.Items.Count = 0 Then
            cmb_PortName.Items.Add("No ports")
        End If
        cmb_PortName.SelectedIndex = 0
    End Sub
    Private Sub InitComboDatabits()
        cmb_DataBits.Items.Clear()
        cmb_DataBits.Items.Add("8")
        cmb_DataBits.Items.Add("7")
        cmb_DataBits.SelectedIndex = 0
    End Sub
    Private Sub InitComboParity()
        cmb_Parity.Items.Clear()
        For Each s As String In [Enum].GetNames(GetType(Parity))
            cmb_Parity.Items.Add(s)
        Next
        cmb_Parity.SelectedIndex = 0
    End Sub
    Private Sub InitComboStopBits()
        cmb_StopBits.Items.Clear()
        For Each s As String In [Enum].GetNames(GetType(StopBits))
            cmb_StopBits.Items.Add(s)
        Next
        cmb_StopBits.SelectedIndex = 1
    End Sub
    Private Sub InitComboHandshake()
        cmb_Handshake.Items.Clear()
        For Each s As String In [Enum].GetNames(GetType(Handshake))
            cmb_Handshake.Items.Add(s)
        Next
        cmb_Handshake.SelectedIndex = 0
    End Sub

    ' ===========================================================================================
    '  CROSS-THREAD UPDATE OF THE FORM CONTROLS
    ' ===========================================================================================
    Delegate Sub Update_RXData_Delegate(ByVal [val] As Integer)
    Private Sub UpdateRxData(ByVal dataReceived As Integer)
        If InvokeRequired Then
            Dim d As New Update_RXData_Delegate(AddressOf UpdateRxData)
            Invoke(d, New Object() {dataReceived})
        Else
            lbl_RxData.Text = CStr(CInt(Val(lbl_RxData.Text)) + dataReceived)
        End If
    End Sub

    Delegate Sub Update_TXData_Delegate(ByVal [val] As Integer)
    Private Sub UpdateTxData(ByVal dataTransmitted As Integer)
        If InvokeRequired Then
            Dim d As New Update_TXData_Delegate(AddressOf UpdateTxData)
            Invoke(d, New Object() {dataTransmitted})
        Else
            lbl_TxData.Text = CStr(CInt(Val(lbl_TxData.Text)) + dataTransmitted)
        End If
    End Sub

    Delegate Sub Update_Status_Delegate(ByVal str As String)
    Private Sub UpdateStatus(ByVal str As String)
        If InvokeRequired Then
            Dim d As New Update_Status_Delegate(AddressOf UpdateStatus)
            Invoke(d, New Object() {str})
        Else
            txt_Status.Text = str
        End If
    End Sub

    ' ===========================================================================================
    '  SERVER and CLIENT buttons
    ' ===========================================================================================
    Private Sub Server_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_Server.Click
        If sockServer Is Nothing Then
            CloseClient()
            OpenServer()
        Else
            CloseServer()
        End If
    End Sub

    Private Sub Client_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_Client.Click
        If sockClient Is Nothing Then
            CloseServer()
            OpenClient()
        Else
            CloseClient()
        End If
    End Sub

    Private Sub OpenServer()
        If sockServer IsNot Nothing Then CloseServer()
        sockServer = New TcpServer(txt_ServerAddr.Text, _
                                            CInt(Val(txt_ServerPort.Text)))
        sockServer.m_SendTimeout = CInt(Val(txt_ServerTimeout.Text))
        sockServer.m_fNoDelay = chk_ServerNoDelay.Checked
        sockServer.Start()
        btn_Server.BackColor = Color.Orange
        btn_Client.BackColor = Color.FloralWhite
    End Sub

    Private Sub CloseServer()
        If sockServer Is Nothing Then Return
        sockServer.StopListener()
        sockServer = Nothing
        btn_Server.BackColor = Color.FloralWhite
        btn_Client.BackColor = Color.FloralWhite
    End Sub

    Private Sub OpenClient()
        If sockClient IsNot Nothing Then CloseClient()
        sockClient = New TcpClient(txt_ClientAddr.Text, _
                                            CInt(Val(txt_ClientPort.Text)))
        sockClient.m_SendTimeout = CInt(Val(txt_ClientTimeout.Text))
        sockClient.m_fNoDelay = chk_ClientNoDelay.Checked
        sockClient.Start()
        btn_Server.BackColor = Color.FloralWhite
        btn_Client.BackColor = Color.Orange
    End Sub

    Private Sub CloseClient()
        If sockClient Is Nothing Then Return
        sockClient.StopClient()
        sockClient = Nothing
        btn_Server.BackColor = Color.FloralWhite
        btn_Client.BackColor = Color.FloralWhite
    End Sub


    ' ===========================================================================================
    '  SERVER 
    ' ===========================================================================================
    Private Sub sockServer_dataReceived(ByVal buffer() As Byte, ByVal bytesReceived As Integer) Handles sockServer.dataReceived
        UpdateRxData(bytesReceived)
        If ComPort.IsOpen Then
            ComPort.Write(buffer, 0, bytesReceived)
        End If
    End Sub

    Private Sub sockServer_dataSent(ByVal buffer() As Byte, ByVal bytesSent As Integer) Handles sockServer.dataSent
        UpdateTxData(bytesSent)
    End Sub

    Private Sub sockServer_logEntry(ByVal strData As String) Handles sockServer.logEntry
        Debug.Print(strData)
        UpdateStatus(strData)
    End Sub

    ' ===========================================================================================
    '  CLIENT
    ' ===========================================================================================
    Private Sub sockClient_dataReceived(ByVal buffer() As Byte, ByVal bytesReceived As Integer) Handles sockClient.dataReceived
        UpdateRxData(bytesReceived)
        If ComPort.IsOpen Then
            ComPort.Write(buffer, 0, bytesReceived)
        End If
    End Sub

    Private Sub sockClient_serverNotReachable(ByVal sSocket As System.Net.Sockets.Socket) Handles sockClient.serverNotReachable
        sockClient.RetryConnection()
    End Sub

    Private Sub sockClient_dataSent(ByVal buffer() As Byte, ByVal bytesSent As Integer) Handles sockClient.dataSent
        UpdateTxData(bytesSent)
    End Sub

    Private Sub sockclient_logEntry(ByVal strData As String) Handles sockClient.logEntry
        Debug.Print(strData)
        UpdateStatus(strData)
    End Sub


    ' ===========================================================================================
    '  COM PORT
    ' ===========================================================================================
    Private Sub cmb_PortName_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_PortName.DropDown
        InitComboPortNames()
    End Sub
    Private Sub cmb_PortName_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_PortName.DropDownClosed
        If Not EventsAreEnabled Then Return
        ComPort_Init()
    End Sub

    Private Sub ComPort_Init()
        ComPort.Close()
        SetComOptions()
        Try
            ComPort.Open()
        Catch
        End Try
        If ComPort.IsOpen Then
            cmb_PortName.BackColor = Color.White
        Else
            cmb_PortName.BackColor = Color.FromArgb(255, 150, 60)
        End If
    End Sub

    Private Sub SetComOptions()
        ComPort.PortName = cmb_PortName.Text
        Dim baud As Int32 = CInt(Val(txt_BaudRate.Text))
        If baud < 1 Or baud > 99000000 Then
            baud = 9600
            txt_BaudRate.Text = baud.ToString
        End If
        ComPort.BaudRate = baud
        ComPort.DataBits = CInt(Val(cmb_DataBits.Text))
        ComPort.Parity = CType([Enum].Parse(GetType(Parity), cmb_Parity.Text), Parity)
        ComPort.StopBits = CType([Enum].Parse(GetType(StopBits), cmb_StopBits.Text), StopBits)
        ComPort.Handshake = CType([Enum].Parse(GetType(Handshake), cmb_Handshake.Text), Handshake)
        ComPort.ReadTimeout = -1
        ComPort.WriteTimeout = -1
    End Sub

    Private Sub serialPort_DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles ComPort.DataReceived
        '
        Dim buffer(5000) As Byte
        Dim index As Integer = 0
        '
        While ComPort.BytesToRead > 0
            buffer(index) = CByte(ComPort.ReadByte())
            index = index + 1
        End While
        '
        If sockServer IsNot Nothing Then
            sockServer.SendData(buffer, index)
        End If
        '
        If sockClient IsNot Nothing Then
            sockClient.SendData(buffer, index)
        End If
    End Sub

    ' ===========================================================================================
    '  VIRTUAL COM - com0com setup
    ' ===========================================================================================
    Private Sub btn_VirtualCom_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_VirtualCom.Click
        Dim SetupPath As String = "C:\Program Files (x86)\com0com\setupg.exe"
        If Not IO.File.Exists(SetupPath) Then SetupPath = "C:\Program Files\com0com\setupg.exe"
        If IO.File.Exists(SetupPath) Then
            Process.Start(SetupPath)
        Else
            Process.Start(Application.StartupPath + "\com0com\Installers")
        End If
    End Sub

End Class
