﻿Imports System
Imports System.Net
Imports System.Net.Sockets
Imports System.Threading
Imports System.Text

Public Class TcpClient

    Private m_Socket As Socket

    Private m_iPort As Integer
    Private m_strIPAddress As String

    Public Event logEntry(ByVal strData As String)
    Public Event dataReceived(ByVal buffer() As Byte, ByVal bytesReceived As Integer)
    Public Event dataSent(ByVal buffer() As Byte, ByVal bytesSent As Integer)
    Public Event serverNotReachable(ByVal sSocket As Socket)

    Public m_SendTimeout As Integer = 10000
    Public m_fNoDelay As Boolean = True

    Public Sub RetryConnection()
        StartClient()
    End Sub

    Public Sub New(ByVal strIPAddress As String, ByVal iPort As Integer)
        m_iPort = iPort
        m_strIPAddress = strIPAddress
    End Sub

    Public Sub Start()
        StartClient()
    End Sub

    ' -------------------------------------------------------------------------
    '  Connect to a remote device.
    ' -------------------------------------------------------------------------
    Private Sub StartClient()
        Try
            '  --------------------------------------------------------------------------------------- Create a TCP/IP socket.
            m_Socket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
            m_Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.SendTimeout, m_SendTimeout)
            m_Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.NoDelay, m_fNoDelay)
            ' ---------------------------------------------------------------------------------------- Connect to the remote endpoint.
            RaiseEvent logEntry("Client is attempting connection...")
            m_Socket.BeginConnect(m_strIPAddress, m_iPort, AddressOf onConnectSuccesfull, m_Socket)
        Catch e As Exception
            RaiseEvent logEntry(e.ToString())
        End Try
    End Sub

    Private Sub onConnectSuccesfull(ByVal ar As IAsyncResult)
        ' -------------------------------------------------------- Retrieve the socket from the state object.
        Dim client As Socket = CType(ar.AsyncState, Socket)
        Try
            ' ---------------------------------------------------- Complete the connection.
            client.EndConnect(ar)
            ' ---------------------------------------------------- Signal that the connection has been made.
            RaiseEvent logEntry("Client connected to " + client.RemoteEndPoint.ToString())
            WaitForData()
        Catch e As Exception
            If client.Connected Then
                RaiseEvent logEntry(e.ToString())
            Else
                RaiseEvent serverNotReachable(client)
            End If
        End Try
    End Sub

    Private Sub WaitForData()
        Try
            Dim state As StateObject = New StateObject()
            state.workSocket = m_Socket
            If m_Socket.Connected Then
                m_Socket.BeginReceive(state.buffer, 0, state.BufferSize, 0, New AsyncCallback(AddressOf onDataReceived), state)
            Else
                RaiseEvent serverNotReachable(m_Socket)
            End If
        Catch ex As Exception
            If m_Socket.Connected Then
                RaiseEvent logEntry(ex.ToString)
            Else
                RaiseEvent serverNotReachable(m_Socket)
            End If
        End Try
    End Sub
    Public Sub onDataReceived(ByVal ar As IAsyncResult)
        '
        Dim state As StateObject = CType(ar.AsyncState, StateObject)
        Dim handler As Socket = state.workSocket
        Try
            Dim content As String = ""
            ' -------------------------------------- Retrieve the state object and the handler socket from the asynchronous state object
            Dim bytesRead As Integer
            If handler.Connected Then
                ' ---------------------------------- Read data from client socket
                bytesRead = handler.EndReceive(ar)
            Else
                RaiseEvent serverNotReachable(handler)
                Exit Sub
            End If
            If bytesRead > 0 Then
                RaiseEvent dataReceived(state.buffer, bytesRead)
            End If
            WaitForData()
        Catch ex As Exception
            If (handler.Connected) Then
                RaiseEvent logEntry(ex.ToString)
            Else
                RaiseEvent serverNotReachable(handler)
            End If
        End Try
    End Sub

    Public Sub SendData(ByVal buffer() As Byte, ByVal bytesToWrite As Integer)
        If m_Socket IsNot Nothing AndAlso m_Socket.Connected Then
            Send(buffer, bytesToWrite)
        Else
            RaiseEvent logEntry("Client is unable to send data, no connection to server!")
        End If
    End Sub

    Private Sub Send(ByVal buffer() As Byte, ByVal bytesToWrite As Integer)
        ' --------------------------------------------------- Convert the string data to byte data using ASCII encoding
        Dim curSocket As Socket
        curSocket = Nothing
        Try
            If m_Socket.Connected Then
                m_Socket.Send(buffer, bytesToWrite, SocketFlags.None)
                RaiseEvent dataSent(buffer, bytesToWrite)
            Else
                RaiseEvent serverNotReachable(curSocket)
            End If
        Catch ex As Exception
            'If curSocket.Connected Then
            '    RaiseEvent logEntry(ex.ToString)
            'Else
            '    RaiseEvent serverNotReachable(curSocket)
            'End If
        End Try
    End Sub

    Public Sub StopClient()
        If m_Socket.Connected Then
            m_Socket.Shutdown(SocketShutdown.Both)
            m_Socket.Close()
            RaiseEvent logEntry("Client closed")
        End If
    End Sub

End Class