﻿Imports System
Imports System.Net
Imports System.Net.Sockets
Imports System.Text
Imports System.Threading

' ----------------------------------------------- State object for reading client data asynchronously
Public Class StateObject
    Public workSocket As Socket = Nothing   ' Client socket.
    Public BufferSize As Integer = 256      ' Receive buffer size
    Public buffer(256) As Byte              ' Receive buffer
    Public sb As New StringBuilder()        ' Received data string
End Class

Public Class TcpServer

    Private m_mainSocket As Socket
    Private m_activeSockets() As Socket
    Private m_clientCount As Integer = 0

    Private m_iPort As Integer
    Private m_strIPAddress As String

    Public Event logEntry(ByVal strData As String)
    Public Event dataReceived(ByVal buffer() As Byte, ByVal bytesRecived As Integer)
    Public Event dataSent(ByVal buffer() As Byte, ByVal bytesSent As Integer)
    Public Event clientNotReachable(ByVal sSocket As Socket)

    Public m_SendTimeout As Integer
    Public m_fNoDelay As Boolean

    Public Sub New(ByVal strIPAddress As String, ByVal iPort As Integer)
        m_iPort = iPort
        m_strIPAddress = strIPAddress
    End Sub

    Public Sub Start()
        StartListening()
    End Sub

    Private Sub StartListening()
        ' ----------------------------------------------------------- Intialize a TCP/IP socket
        m_mainSocket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
        m_mainSocket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.SendTimeout, m_SendTimeout)
        m_mainSocket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.NoDelay, m_fNoDelay)
        '
        ' ----------------------------------------------------------- Bind socket to local endpoint and listen for incoming connections
        Dim localEndPoint As New IPEndPoint(IPAddress.Any, m_iPort)
        Try
            m_mainSocket.Bind(localEndPoint)
            m_mainSocket.Listen(4)
            RaiseEvent logEntry("Waiting for a connection...")
            m_mainSocket.BeginAccept(New AsyncCallback(AddressOf onClientConnect), 0)
        Catch e As Exception
            RaiseEvent logEntry(e.ToString())
        End Try
    End Sub

    Public Sub onClientConnect(ByVal ar As IAsyncResult)
        Try
            ReDim Preserve m_activeSockets(0 To m_clientCount + 1)
            m_activeSockets(m_clientCount) = m_mainSocket.EndAccept(ar)
            '
            RaiseEvent logEntry("Client connected: " + m_activeSockets(m_clientCount).RemoteEndPoint.ToString)
            '
            WaitForData(m_activeSockets(m_clientCount))
            m_clientCount = m_clientCount + 1
            m_mainSocket.BeginAccept(New AsyncCallback(AddressOf onClientConnect), 0)
        Catch ex As Exception
            RaiseEvent logEntry(ex.ToString)
        End Try
    End Sub

    Private Sub WaitForData(ByVal sSocket As Socket)
        Try
            ' ------------------------------------------- Create the state object
            Dim state As StateObject = New StateObject()
            state.workSocket = sSocket
            If sSocket.Connected Then
                sSocket.BeginReceive(state.buffer, 0, state.BufferSize, 0, New AsyncCallback(AddressOf onDataReceived), state)
            Else
                RaiseEvent clientNotReachable(sSocket)
            End If
        Catch ex As Exception
            If sSocket.Connected Then
                RaiseEvent logEntry(ex.ToString)
            Else
                RaiseEvent clientNotReachable(sSocket)
            End If
        End Try
    End Sub

    Public Sub onDataReceived(ByVal ar As IAsyncResult)
        '
        Dim state As StateObject = CType(ar.AsyncState, StateObject)
        Dim handler As Socket = state.workSocket
        Try
            ' ------------------------------------------- Read data from client socket 
            Dim content As String = ""
            Dim bytesRead As Integer
            '
            If handler.Connected Then
                bytesRead = handler.EndReceive(ar)
            Else
                RaiseEvent clientNotReachable(handler)
                Exit Sub
            End If
            '
            If bytesRead > 0 Then
                RaiseEvent dataReceived(state.buffer, bytesRead)
            End If
            '
            WaitForData(handler)
            '
        Catch ex As Exception
            If handler.Connected Then
                RaiseEvent logEntry(ex.ToString)
            Else
                RaiseEvent clientNotReachable(handler)
            End If
        End Try
    End Sub

    Public Sub StopListener()
        m_mainSocket.Close()
        m_mainSocket = Nothing
        RaiseEvent logEntry("Server closed")
    End Sub

    Public Sub SendData(ByVal buffer() As Byte, ByVal bytesToWrite As Integer)
        If m_clientCount > 0 Then
            Send(buffer, bytesToWrite)
        Else
            RaiseEvent logEntry("Server is unable to send data, no clients connected!")
        End If
    End Sub

    Private Sub Send(ByVal buffer() As Byte, ByVal bytesToWrite As Integer)
        ' ------------------------------------------------Convert the string data to byte data using ASCII encoding
        Dim curSocket As Socket = Nothing

        Try
            For Each socket As Socket In m_activeSockets
                curSocket = socket
                If socket IsNot Nothing Then
                    If socket.Connected Then
                        socket.Send(buffer, bytesToWrite, SocketFlags.None)
                        RaiseEvent dataSent(buffer, bytesToWrite)
                    Else
                        RaiseEvent clientNotReachable(curSocket)
                    End If
                End If
            Next
        Catch ex As Exception
            If (curSocket.Connected) Then
                RaiseEvent logEntry(ex.ToString)
            Else
                RaiseEvent clientNotReachable(curSocket)
            End If
        End Try
    End Sub

    Private Sub AsynchronousSocketListener_clientNotReachable(ByVal sSocket As Socket) Handles Me.clientNotReachable
        RaiseEvent logEntry("Client not reachable: " + sSocket.RemoteEndPoint.ToString)
    End Sub

End Class